//------------------------------------------------------------------------
// GraphWaves Applet
// This applet draws one kind of sound waves and makes it possible to
// adjust graph's phase, amplitude and frequency. If user clicks on the
// 'Play' button, it will generate 1 second length 8 bit 8000Hz sampling
//  mu-law sound.
//
// INPUT:  HTML parameter of graph kind and user mouse clicking
//
// OUTPUT: graph and 1 second length 8-bit mu-law audio sound.
//
// @author Jungkee Kim

import java.applet.*;
import java.awt.*; 
import java.lang.Math; 
import java.util.*;

import interfaces.AppletBase;
import messages.*;
import consts.Const;

public class GraphWaves extends AppletBase {
  GraphDraw           soundgraph;              // sound graph
  PlayControlPanel    control;                 // control panel
  String              wave;                    // kind of waves

  public void init() {
    register();
    wave = getParameter("wave");
    if (wave == null)                          // if no parameter passed
      wave = "Sine";
    setLayout(new BorderLayout(2, 2));
    control = new PlayControlPanel(this);
    soundgraph = new GraphDraw(this);
    add("North", control);
    add("Center", soundgraph);
    if (wave.equals("Sine")) soundgraph.setSine();
    else if (wave.equals("Rectangular")) soundgraph.setRectangular();
    else if (wave.equals("Triangle")) soundgraph.setTriangle();
    control.setBackground(new Color(0xFFEBCD));
  }

  public void receive (AppEventMessage em) {
    try {
      if (em.getType() == Const.DATA) {
        byte[] act = em.getData();
        if ( act[0] == 1 )
	  control.PlaySound();
	else if ( act[0] == 2 )
	  soundgraph.paper.update_amplitude(1);
	else if ( act[0] == 3 )
	  soundgraph.paper.update_amplitude(2);

	switch ( act[1] ) {
	case 1 :
	  soundgraph.paper.set_frequency(1);
	  break;
	case 2 :
	  soundgraph.paper.set_frequency(2);
	  break;
	case 3 :
	  soundgraph.paper.set_frequency(3);
	  break;
	case 4 :
	  soundgraph.paper.set_frequency(4);
	  break;
	case 5 :
	  soundgraph.paper.set_frequency(5);
	  break;
	case 6 :
	  soundgraph.paper.set_frequency(6);
	  break;
	case 7 :
	  soundgraph.paper.set_frequency(7);
	  break;
	default : break;
	}
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }
}


/* This panel draws play button panel and generates sound when clicking */
class PlayControlPanel extends Panel {
  GraphWaves parent;

  PlayControlPanel(GraphWaves parent) {
    this.parent = parent;
    setLayout(new BorderLayout());
    add("Center", new Button("Play"));
  }

  public boolean action(Event evt, Object arg) {
    if (evt.target instanceof Button) {
      getWave((String)arg);
      return true;
    }
    else return false;
  }

  void getWave(String cname) {
    byte[] act = new byte[2];
    act[0] = 0;
    act[1] = 0;

    if (cname.equals("Play")) {
      act[0] = 1;
      AppEventMessage em = new AppEventMessage(Const.DATA, act);
      parent.send(em);
      PlaySound();
    }
  }

  void PlaySound() {
    SecondSound stream = new SecondSound(this);
    stream.run();
  }
}
