//------------------------------------------------------------------------
// SecondSound class
// This class makes 1 second length sounds. The sound format is
// CCITT G. 711 8-bit 8000Hz sampling mu-law. First, the sound
// storage, buffer has the header part data initialization.
// In this class, the header part is 40 byte length.
// Next, the graph height of the sound wave has been read from the
// other class. It will be converted as 16 bit linear value. I assumed
// the center as 0 value, upper part as minus value and lower part
// as plus value. Due to the graphic coordinate system, the graph
// is reversed amplitude unlike common math graphs. However, it does not
// matter to the sound generating and human cannot feel the difference
// millisecond phase shift. The linear2ulaw method is revised from
// Craig F. Reese's code on the mu-law FAQ.
//
// INPUT: graph height measure function
//
// OUTPUT: 1 second length 8-bit mu-law audio sound.
//
// @author Jungkee Kim
// @version 1.0, May 1997

import java.lang.*;
import java.io.*;
import java.applet.*;
import java.awt.*;
import sun.audio.*;

class SecondSound extends Applet implements Runnable {
  PlayControlPanel parent;                      // parent class
  public static Thread thread;                  // thread
  public static byte buffer[] = new byte[8040]; // audio data
  public static int exp_lut[] = {0,0,1,1,2,2,2,2,3,3,3,3,3,3,3,3,
				 4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
				 5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
				 5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
				 6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
				 6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
				 6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
				 6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
				 7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
				 7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
				 7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
				 7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
				 7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
				 7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
				 7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
				 7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7};

  SecondSound(PlayControlPanel parent) {
    this.parent = parent;
    /* header part of mu-law file */
    byte info[] = {46, 115, 110, 100, 0, 0, 0, 40,
		0, 0, 31, 64, 0, 0, 0, 1,
		0, 0, 31, 64, 0, 0, 0, 1,
		74, 46, 32, 75, 105, 109, 39, 115,
		0, 0, 0, 0, 0, 0, 0, 0 };
    for(int i = 0; i<40; i++) {
      buffer[i] = info[i];
    }
    for(int i = 0; i<8000; i++) {
      int h = (int)parent.parent.soundgraph.paper.f(i);
      buffer[i + 40] = linear2ulaw(get16Linear(h));
    }
  }

  public void start() {
    if (thread == null) {
      thread = new Thread(this);
      thread.start();
    }
  }

  public void stop() {
    if (thread != null) {
      thread.stop();
      thread = null;
    }
  }

  public void run() {
      AudioData audiodata = new AudioData(buffer);
      AudioDataStream audioStream = new AudioDataStream(audiodata);
      AudioPlayer.player.start(audioStream);
  }

  /* Converting the graph height to 16 bit linear value. 2^16 / 2 = 32768 */
  int get16Linear(int h) {
    int height = parent.parent.soundgraph.paper.size().height;
    if ( h > height / 2 ) {
      return (h - height / 2) * 32768 / (height / 2);
    }
    else {
      return (height / 2 - h) * -32768 / (height / 2);
    }
  }

  /* Revised from Reese's code */
  byte linear2ulaw(int sample) {
    int sign, exponent, mantissa;
    byte ulawbyte;

    /* get the sample into sign-magnitude */
    sign = (sample >> 8) & 0x80;          // set aside the sign
    if (sign != 0) sample = -sample;      // get magnitude 
    if (sample > 32635) sample = 32635;   // clip the magnitude
    /* convert from 16 bit linear to ulaw */
    sample = sample + 0x84;
    exponent = exp_lut[(sample>>7) & 0xFF];
    mantissa = (sample >> (exponent+3)) & 0x0F;
    ulawbyte = (byte)~(sign | (exponent << 4) | mantissa);
    if (ulawbyte == 0 ) ulawbyte = 0x02;
    return(ulawbyte);
  }
}
